"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ObjectType_1 = require("../types/ObjectType");
const ObjectModule_1 = __importDefault(require("./ObjectModule"));
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
class Feature {
    constructor(featureId, meta) {
        this.featureId = featureId;
        this.meta = meta;
    }
    //
    // Static methods
    //
    static create(params = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: params.featureId,
                },
                meta: params.meta,
            }, options);
            return new Feature(response.objectId, response.meta);
        });
    }
    static get(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.get({
                objectType: ObjectType_1.ObjectType.Feature,
                objectId: featureId,
            }, options);
            return new Feature(response.objectId, response.meta);
        });
    }
    static update(featureId, params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.update({
                objectType: ObjectType_1.ObjectType.Feature,
                objectId: featureId,
            }, { meta: params.meta }, options);
            return new Feature(response.objectId, response.meta);
        });
    }
    static delete(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return ObjectModule_1.default.delete({
                objectType: ObjectType_1.ObjectType.Feature,
                objectId: featureId,
            }, options);
        });
    }
    static listFeatures(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield ObjectModule_1.default.list(Object.assign({ objectType: ObjectType_1.ObjectType.Feature }, listParams), options);
                const features = response.results.map((object) => new Feature(object.objectId, object.meta));
                return Object.assign(Object.assign({}, response), { results: features });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listFeaturesForPricingTier(pricingTierId, listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const queryResponse = yield WarrantModule_1.default.query(`select feature where pricing-tier:${pricingTierId} is *`, listParams, options);
                const features = queryResponse.results.map((queryResult) => new Feature(queryResult.objectId, queryResult.meta));
                return Object.assign(Object.assign({}, queryResponse), { results: features });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignFeatureToPricingTier(pricingTierId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                }
            }, options);
        });
    }
    static removeFeatureFromPricingTier(pricingTierId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                }
            }, options);
        });
    }
    static listFeaturesForTenant(tenantId, listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const queryResponse = yield WarrantModule_1.default.query(`select feature where tenant:${tenantId} is *`, listOptions, options);
                const features = queryResponse.results.map((queryResult) => new Feature(queryResult.objectId, queryResult.meta));
                return Object.assign(Object.assign({}, queryResponse), { results: features });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignFeatureToTenant(tenantId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                }
            }, options);
        });
    }
    static removeFeatureFromTenant(tenantId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                }
            }, options);
        });
    }
    static listFeaturesForUser(userId, listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const queryResponse = yield WarrantModule_1.default.query(`select feature where user:${userId} is *`, listOptions, options);
                const features = queryResponse.results.map((queryResult) => new Feature(queryResult.objectId, queryResult.meta));
                return Object.assign(Object.assign({}, queryResponse), { results: features });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignFeatureToUser(userId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    static removeFeatureFromUser(userId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.Feature;
    }
    getObjectId() {
        return this.featureId;
    }
}
exports.default = Feature;
