"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Authorization_1 = __importDefault(require("./Authorization"));
const Feature_1 = __importDefault(require("./Feature"));
const ObjectModule_1 = __importDefault(require("./ObjectModule"));
const PricingTier_1 = __importDefault(require("./PricingTier"));
const User_1 = __importDefault(require("./User"));
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
const ObjectType_1 = require("../types/ObjectType");
class Tenant {
    constructor(tenantId, meta) {
        this.tenantId = tenantId;
        this.meta = meta;
    }
    //
    // Static methods
    //
    static create(params = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: params.tenantId,
                },
                meta: params.meta,
            }, options);
            return new Tenant(response.objectId, response.meta);
        });
    }
    static batchCreate(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.batchCreate(params.map((tenant) => ({
                object: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenant.tenantId,
                },
                meta: tenant.meta,
            })), options);
            return response.map((object) => new Tenant(object.objectId, object.meta));
        });
    }
    static get(tenantId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.get({
                objectType: ObjectType_1.ObjectType.Tenant,
                objectId: tenantId,
            }, options);
            return new Tenant(response.objectId, response.meta);
        });
    }
    static update(tenantId, params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.update({
                objectType: ObjectType_1.ObjectType.Tenant,
                objectId: tenantId,
            }, { meta: params.meta }, options);
            return new Tenant(response.objectId, response.meta);
        });
    }
    static delete(tenantId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return ObjectModule_1.default.delete({
                objectType: ObjectType_1.ObjectType.Tenant,
                objectId: tenantId,
            }, options);
        });
    }
    static batchDelete(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield ObjectModule_1.default.batchDelete(params.map((tenant) => ({
                object: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenant.tenantId,
                },
            })), options);
        });
    }
    static listTenants(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.list(Object.assign({ objectType: ObjectType_1.ObjectType.Tenant }, listParams), options);
            const tenants = response.results.map((object) => new Tenant(object.objectId, object.meta));
            return Object.assign(Object.assign({}, response), { results: tenants });
        });
    }
    static listTenantsForUser(userId, listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const queryResponse = yield WarrantModule_1.default.query(`select tenant where user:${userId} is *`, listParams, options);
            const tenants = queryResponse.results.map((queryResult) => new Tenant(queryResult.objectId, queryResult.meta));
            return Object.assign(Object.assign({}, queryResponse), { results: tenants });
        });
    }
    //
    // Instance methods
    //
    listUsers(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return User_1.default.listUsersForTenant(this.tenantId, listParams, options);
        });
    }
    assignUser(userId, role, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return User_1.default.assignUserToTenant(this.tenantId, userId, role, options);
        });
    }
    removeUser(userId, role, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return User_1.default.removeUserFromTenant(this.tenantId, userId, role, options);
        });
    }
    listPricingTiers(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.listPricingTiersForTenant(this.tenantId, listParams, options);
        });
    }
    assignPricingTier(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.assignPricingTierToTenant(this.tenantId, pricingTierId, options);
        });
    }
    removePricingTier(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.removePricingTierFromTenant(this.tenantId, pricingTierId, options);
        });
    }
    listFeatures(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.listFeaturesForTenant(this.tenantId, listParams, options);
        });
    }
    assignFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.assignFeatureToTenant(this.tenantId, featureId, options);
        });
    }
    removeFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.removeFeatureFromTenant(this.tenantId, featureId, options);
        });
    }
    hasFeature(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasFeature({ featureId: params.featureId, subject: { objectType: ObjectType_1.ObjectType.Tenant, objectId: this.tenantId }, context: params.context }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.Tenant;
    }
    getObjectId() {
        return this.tenantId;
    }
}
exports.default = Tenant;
